@@ -39,7 +39,7 @@ def test_nbadapter(test_notebook, morph_notebook, caplog):
     for k, v in parameters.items():
         print("\033[31m", k, ":", v, "\033[0m")
 
-    assert len(parameters) == 5
+    assert len(parameters) == 6
 
     assert 'comment' in parameters['scwid']
     assert parameters['scwid']['owl_type'] == "http://odahub.io/ontology/integral#ScWID"
@@ -73,7 +73,7 @@ def test_nbadapter(test_notebook, morph_notebook, caplog):
 
 
 def test_nbadapter_repo(test_notebook_repo):
-    from nb2workflow.nbadapter import NotebookAdapter, find_notebooks
+    from nb2workflow.nbadapter import NotebookAdapter, find_notebooks, validate_oda_dispatcher
 
     nbas = find_notebooks(test_notebook_repo)
 
@@ -85,7 +85,7 @@ def test_nbadapter_repo(test_notebook_repo):
         parameters = nba.extract_parameters()
 
         print(parameters)
-        assert len(parameters) == 5
+        assert len(parameters) == 6
 
         if os.path.exists(nba.output_notebook_fn):
             os.remove(nba.output_notebook_fn)
@@ -102,6 +102,8 @@ def test_nbadapter_repo(test_notebook_repo):
         assert len(output) == 4
         assert 'spectrum' in output
 
+        validate_oda_dispatcher(nba)
+
 
 def test_nbreduce(test_notebook):
     from nb2workflow.nbadapter import NotebookAdapter, nbreduce, setup_logging
